;                               BusOneWire.INC         
;             Routines de gestion du protocole ONE WIRE de chez DALLAS
;                          Routines de niveau 1 et 2
;
; 	   	  !!! Attention tempo calcule en fonction du quartz de 4Mhz !!!                       
;
; Description des Routines :
; BUS_OUT		: configuration du bus en sortie
; BUS_IN		: configuration du bus en entre
; INIT  		: Effectue un reset sur le bus
; READ  		: Effectue un "read slot"
; WRITE1S 		: Effectue un "write 1 slot"
; WRITE0S 		: Effectue un "write 0 slot"
; WRITOCT 		: Ecrit l'octet contenu dans buffer sur le bus ONE WIRE
; READOCT 		: Ecrit un octet du SCRATCHPAD dans la variable buffer
;
; ********************************************************************************
; *								ROUTINES DE NIVEAU 1							 *
; ********************************************************************************
; Cette routine configure le BUS en sortie (Capteur ds18b20 sur RA5)
BUS_OUT 					; du matre vers l'esclave
	movlw	b'00000000'		; RA5 configure en sortie (1=entre)
	movwf	TRISA
	return					; retour

; Cette routine configure le BUS en entre	
BUS_IN 						; de l'esclave vers le matre
	movlw	b'00100000'		; RA5 configure en entre (1=entre)
	movwf	TRISA
	return					; le BUS est en attente d'un signal de l'esclave

; ********************************************************************************
; *	I)-- SEQUENCE D'INITIALISATION							                     *
; ********************************************************************************
; Le matre envoie le RESET PULSE en forant le BUS  0 pendant au moins 480s
; puis il relche le BUS et passe en mode rception. L'esclave dtecte le front 
; montant au moment du relchement du BUS par le matre et doit rpondre dans les
; 60s qui suivent en maintenant  son tour le BUS  l'tat bas	en renvoyant un
; PRESENCE PULSE pendant un temps 60s < t < 240s.
; ATTENTION : la squence complte d'initialisation doit durer au moins 960s
							 	 
INIT
	call	BUS_OUT			; on rgle le BUS en sortie
	bcf		BUS_DS18S20		; tat bas = forc                           (0V)
    call	tempo480		; on temporise 480 S
	bsf		BUS_DS18S20		; on relche le BUS
	call	BUS_IN			; on le passe en entre
	call	tempo60			; l'esclave doit rpondre dans les 60s 
	btfsc	BUS_DS18S20		; On teste donc si le BUS est bien forc
	goto	INIT			; si non, on rinitialise
	call	tempo480		; si oui l'esclave fait sa rponse en maintenant le BUS
							; bas (=0) ; on ne temporise pas pendant 240s mais 
							; pendant 480S pour atteindre les 960s indispensables		
	return					

; ********************************************************************************
; *	                          ROUTINES DE NIVEAU 1                               *
; ********************************************************************************
; * II)-- COMMANDES ROM =  routines de gestion du BUS				             *
; ********************************************************************************
; Se souvenir que le BUS ONE-WIRE fonctionne par SLOT de 60s et RECOVERY de 1s
; ********************************************************************************
; READ TIME SLOT
; Le matre doit maintenir le BUS  0 pendant 1s puis le relcher, lire avant 15s
; le niveau logique sur le BUS et attendre la fin du slot (60s)							 
READ
	clrwdt						
	call	BUS_OUT			; on rgle le BUS en sortie
	bcf		BUS_DS18S20		; le matre impose un niveau bas sur le BUS pendant 
	call	tempo4			; 4s(au lieu d'une)	
	call	BUS_IN			; le matre relche le BUS en le passant en entre  
	btfss	BUS_DS18S20		; on vrifie que le bus a bien t relch retour( 5V)	
	movlw	d'0'			; si=non, on charge '0'	dans w
	btfsc	BUS_DS18S20		; si oui, on reteste le bus (astuce de programmation pour 
							; ne pas ecraser la valeur '0')
	movlw	d'1'			; si le BUS est relch, on remplace le '0' par '1'
	movwf	temporaire		; on stocke la valeur dans la variable 'temporaire'
	call 	tempo60			; on attend jusqu' la fin du slot
	return					; retour

; ********************************************************************************
; WRITE 1 SLOT	
; Le matre doit maintenir le BUS  0 (tat bas) pendant moins de 15s puis 
; le relcher et attendre la fin du slot							 
WRITE1S
	clrwdt
	call	BUS_OUT			; on rgle le BUS en sortie, niveau bas
	bcf		BUS_DS18S20		; on doit imposer un niveau bas sur le BUS pendant
	call	tempo7			; moins de 15s - on prendra 7s(tempo7) 
	bsf		BUS_DS18S20		; on relche le BUS
	call	tempo60			; le recovery (temps de rcupration) est inclus
	return

; ********************************************************************************
; WRITE 0 SLOT
; Le matre doit maintenir le BUS  0 (tat bas) pendant toute la dure du slot 
; on finit sur un recovery bien dimensionn								 
WRITE0S
	clrwdt						
	call	BUS_OUT			; on rgle le BUS en sortie
	bcf		BUS_DS18S20		; on doit imposer un niveau bas sur le BUS pendant
	call	tempo60			; toute la dure du slot (60s)
	bsf		BUS_DS18S20 	; on relche le BUS
	call	tempo4			; recovery time - par scurit on prendra 4s
	return

; ********************************************************************************
; *                              ROUTINES DE NIVEAU2                             *
; ********************************************************************************
; *	LES DEUX ROUTINES QUI SUIVENT AUTOMATISENT LE TRANSFERT EN LECTURE OU EN     *
; * ECRITURE D'UN OCTET COMPLET				                                     *
; ********************************************************************************
; Lecture d'un octet dans le buffer
READOCT
	movlw	D'8'			; initialisation du compteur de bits
	movwf	compteur		; avec la valeur 8 (nombre de bits dans l'octet)
READ_OCT
	call	READ			; on lit le bit courant sur le BUS on a donc
							; soit '0' soit '1' dans 'temporaire'
	movf	temporaire,w	; on rappelle la valeur dans le registre de travail w
	xorlw	d'1'			; on fait un 'ou' exclusif entre la valeur de w et '1'
							; donc si w=0, xor vaut '1' et si w=1, xor vaut '0'
	bcf		STATUS, C		; on initialise le flag C  0
	btfsc	STATUS, Z		; on teste le flag Z qui varie en fonction de 'xor'
	bsf		STATUS, C		; si Z=1 (xor=0), on positionne le flag C (C=1)
	rrcf	buffer, F		; et on fait un dcallage dans le buffer avec le flag C
							; le bit sortant  droite est remplac par la valeur 
							; de 'C' rentrant  gauche
							; si Z=0, on laisse C=0 et on fait un dcallage dans
							; le buffer
	decfsz	compteur		; on dcrmente le compteur pour le bit suivant
	goto	READ_OCT 		; et on boucle
	return

; ********************************************************************************
; Ecriture d'un octet dans le buffer
WRITOCT
	movlw	d'8'			; initialisation du compteur
	movwf	compteur		; avec la valeur 8 (nombre de bits dans l'octet)
WRIT_OCT
	btfsc	buffer, 0		; on teste la valeur du bit 0 de 'buffer' 
	call	WRITE1S			; s'il vaut '1' on choisit le WRITE 1 SLOT
	btfss	buffer, 0		; s'il vaut '0' on fait le test inverse ; comme il ne
	call	WRITE0S			; peut valoir que '0' on choisit le WRITE 0 SLOT
	rrcf	buffer			; on dcale  droite 1 fois la variable pour le bit
	decfsz	compteur		; suivant ; on dcrmente le compteur
	goto	WRIT_OCT		; et on boucle
	return

; ********************************************************************************
; * III)-- COMMANDES FONCTIONS =  fonctions de commande du BUS utilisant les     *
;							      routines de niveau 1				             *
; ********************************************************************************
; Commande de conversion de temprature envoye  l'esclave -- l'esclave lance 
; la conversion unique
CONVERT_T
	call	INIT			; squence d'init
	movlw	H'CC'			; ROM command--SKIP ROM=CCH--1 seul esclave sur le BUS
	movwf	buffer			; le code est charg dans le buffer
	call	WRITOCT			; le code est envoy  l'esclave sur le BUS 	
	movlw	H'44'			; ROM command--CONVERT_T=44H
	movwf	buffer			; le code est charg dans le buffer
	call	WRITOCT			; transmission de la commande  l'esclave sur le BUS	
							; l'esclave rpond par '0' tant que la conversion 
							; n'est pas termin
TEST_CONVERT				; on teste si la conversion est termine
	call	READ			; le matre lance un READ SLOT pour lire l'tat du BUS
							; l'tat du BUS( 0 ou 1) est stock par le READ
	movf	temporaire,w	; dans la variable 'temporaire'puis rappel dans w
	xorlw	d'1'			; On fait un 'ou exclusif' avec '1' et on teste le 
	btfss	STATUS,Z		; bit Z. Le seul cas ou il soit positionn  '1',
							; c'est si le READSLOT vaut '1', c..d quand la
	goto	TEST_CONVERT	; conversion est termin ; sinon on refait le test
	return

; ********************************************************************************
; Commande de lecture du SCRATCHPAD envoye  l'esclave -- lecture de 2 octets
; de temprature du SCRATCHPAD
LEC_TEMP
	call	INIT			; squence d'init
	movlw	H'CC'			; ROM command--SKIP ROM=CCH--
	movwf	buffer			; le code est charg dans le buffer
	call	WRITOCT			; le code est envoy  l'esclave sur le BUS 	
	movlw	H'BE'			; ROM command -- READ SCRATCHPAD=BEH
	movwf	buffer			; le code est charg dans le buffer
	call	WRITOCT			; transmission de la commande  l'esclave sur le BUS
							; l'esclave prpare ces deux octets  renvoyer
	call	READOCT			; on lit le premier octet qu'on place dans 'buffer'
	movf	buffer,w		; on recopie 'buffer' dans w
	movwf	bufferL			; on copie w dans bufferL = octet de poids faible
	call	READOCT			; on lit le deuxime octet qu'on place dans 'buffer'
	movf	buffer,w		; on recopie 'buffer' dans w
	movwf	bufferH			; on copie w dans bufferH = octet de poids fort
	call	INIT			; comme il n'est pas ncessaire de lire tout le 
							; SCRATCHPAD, on arrte la lecture par un RESET 
	return					;

; traitement de la dcimale
DECIMALE
	movlw	b'00110000'		; on charge le code ASCII de '0' dans w
	btfsc	bufferL,3		; on teste le bit 3 de 'bufferL' 
	movlw	b'00110101'		; s'il vaut '1', on remplace '0' par '5' dans w
							; qu'on chargera ensuite dans 'decimale'
	movwf	decimale		; s'il vaut '0', on charge '0' dans 'dcimale' 
	return

; ********************************************************************************
; *                         ROUTINES DE TEMPORISATION                            *
; ********************************************************************************
; Temporisation de 480s  --- voir micro-dlais (fichier excel)
tempo480
	clrwdt
	nop
	movlw	d'159'
	movwf	tempo
tempo_480
	decfsz	tempo,f
	goto	tempo_480
	return
;---------------------------------------------------------------------------------
; Temporisation de 60s
tempo60
	movlw	d'19'
	movwf	tempo
tempo_60
	clrwdt
	nop
	decfsz	tempo,f
	goto	tempo_60
	return
;---------------------------------------------------------------------------------
; Temporisation de 10s
tempo10
	movlw	d'3'
	movwf	tempo
tempo_10
	decfsz	tempo,f
	goto	tempo_10
	nop
	return
;---------------------------------------------------------------------------------
; Temporisation de 7s
tempo7
	movlw	d'2'
	movwf	tempo
tempo_7
	decfsz	tempo,f
	goto	tempo_7
	return
;---------------------------------------------------------------------------------
; Temporisation de 4s
tempo4
	movlw	d'1'
	movwf	tempo
tempo_4
	decfsz	tempo,f
	goto	tempo_4
	return